# Generated by Django 3.0.5 on 2020-04-22 17:06

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users_extended', '0007_extendeduser_alias'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='useraditionalpermission',
            options={'managed': False, 'permissions': (('list_user', 'Can list user'), ('change_password_other_user', 'Can change other user password'), ('change_pin_other_user', 'Can change other user pin'), ('change_qr_other_user', 'Can change other user qr'), ('change_profile_other_user', 'Can change other user profile'), ('change_profile_image_other_user', 'Can change other user profile image'), ('change_profile_user', 'Can change user profile'), ('change_password_user', 'Can change user password'), ('change_pin_user', 'Can change user pin'), ('change_qr_user', 'Can change user qr'), ('change_profile_image_user', 'Can change user profile image'), ('change_permission_user', 'Can change user permission'), ('make_superuser_user', 'Can make user superuser'), ('make_staff_user', 'Can make user staff'), ('make_active_user', 'Can make user active'))},
        ),
    ]
