# Generated by Django 3.0.5 on 2020-04-02 23:47

import app_configurations.models
from django.db import migrations, models
import imagekit.models.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='GeneralInfoConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('application_name', models.CharField(default='App. Name', max_length=50)),
                ('logo', imagekit.models.fields.ProcessedImageField(blank=True, null=True, upload_to=app_configurations.models.GeneralInfoConfiguration.imagen_logo_upload_to)),
            ],
        ),
    ]
