# Generated by Django 3.0.5 on 2020-05-04 18:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('countries', '0006_auto_20200504_1407'),
        ('users_extended', '0010_auto_20200426_0245'),
    ]

    operations = [
        migrations.AddField(
            model_name='extendeduser',
            name='city',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='countries.City'),
        ),
    ]
